#include "MousePicker.h"

#include <QtWidgets>

#include <osg/Point>
#include <osg/MatrixTransform>
#include <osgViewer/Viewer>

MousePicker::MousePicker(QObject *parent)
    : QObject(parent)
{
    // ֹϵӰµ
    currentDrawNode->setNodeMask(0xffffffff & (~0x00000004));
}

void MousePicker::setRoot(osg::ref_ptr<osg::Group> root)
{
//    root->addChild(currentDrawNode);
    root->addChild(drawNodeList);
}

void MousePicker::activate(PickingMode _mode, osg::Node* node)
{
    disactivate();
    mode = _mode;
    isActivated = true;
    model = node;
}

bool MousePicker::handle(const osgGA::GUIEventAdapter &ea, osgGA::GUIActionAdapter &aa)
{
    if(!isActivated){
        return false;
    }
    osgViewer::Viewer* pViewer = dynamic_cast<osgViewer::Viewer*>( &aa );
    osgUtil::LineSegmentIntersector::Intersections intersections;
    pViewer->computeIntersections(ea, intersections, 0x00000004);
    if(intersections.empty()){
        return false;
    }
    osg::Vec3 mouseWorldCoord = intersections.begin()->getWorldIntersectPoint();
    switch( ea.getEventType() ){
    case osgGA::GUIEventAdapter::PUSH:{
        if(ea.getButton() == osgGA::GUIEventAdapter::LEFT_MOUSE_BUTTON){
            if(!isDrawing){
                // ʼlineGeometry
                lineGeometry = new osg::Geometry;
                lineGeometry->getOrCreateStateSet()->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF | osg::StateAttribute::OVERRIDE);
                currentDrawNode->addDrawable(lineGeometry);
                // ʼvertexArray
                vertexArray = new osg::Vec3Array;
                vertexArray->push_back(mouseWorldCoord);
                osg::ref_ptr<osg::Vec4Array>  color = new osg::Vec4Array();
                color->push_back(osg::Vec4(0.0f, 0.8f, 0.8f, 0.5f) );
                lineGeometry->setColorArray(color, osg::Array::BIND_OVERALL);
                isDrawing = true;
            }
            vertexArray->push_back(mouseWorldCoord);
            vertexArray->dirty();
            pushPnt = mouseWorldCoord;
        }
        if(ea.getButton() == osgGA::GUIEventAdapter::RIGHT_MOUSE_BUTTON){
            // Ҽʱ, ˳༭
            disactivate();
        }
        return false;
    }
    case osgGA::GUIEventAdapter::RELEASE:{
        if(ea.getButton() == osgGA::GUIEventAdapter::LEFT_MOUSE_BUTTON){
            // ¼ʱ
            if(pushPnt != mouseWorldCoord){
                // ûϣ϶ӽʱ¼¼װ·
                vertexArray->pop_back();
                return false;
            }
            // Ӧ
//            currentDrawNode->addDrawable(createPointGeode(mouseWorldCoord));

            osg::MatrixTransform* mt = new osg::MatrixTransform;
            mt->setMatrix(
                        osg::Matrix::translate(mouseWorldCoord));
            mt->addChild(model);
            drawNodeList->addChild(mt);
            if(mode == LinePicker){
                // ҪƸ߶βʾ
                lineGeometry->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::LINE_STRIP, 0, vertexArray->size()));
            }
            if(mode == AreaPicker){
                // ҪƸ沢ʾ
                lineGeometry->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::POLYGON, 0, vertexArray->size()));
            }
        }
        return false;
    }
    case osgGA::GUIEventAdapter::MOVE:{
        if(isDrawing){
            // ƹУͬͼͽͼλƵԤ
            vertexArray->pop_back();
            vertexArray->push_back(mouseWorldCoord);
            vertexArray->dirty();
            if(mode == PointPicker){
                return false;
            }
            // MOVEʵʱlineGeometry
            lineGeometry->setVertexArray(vertexArray);
            lineGeometry->dirtyDisplayList();
        }
        return false;
    }
    case osgGA::GUIEventAdapter::DOUBLECLICK:{
        if(isDrawing){
            // ˫ʱɵǰgeodeı༭
            isDrawing = false;
        }
        // ˫ʱӦӽǲʷtrue
        return true;
    }
    }
    return false;
}

void MousePicker::disactivate()
{
    isDrawing = false;
    isActivated = false;
    emit editionDone(mode, vertexArray);

    currentDrawNode->removeChildren(0, currentDrawNode->getNumChildren());
    lineGeometry = nullptr;
    vertexArray = nullptr;
}

osg::ref_ptr<osg::Geometry> MousePicker::createPointGeode(const osg::Vec3 &pos)
{
    osg::Geometry *geom = new osg::Geometry();

    osg::ref_ptr<osg::Vec3Array>  vertex = new osg::Vec3Array();
    vertex->push_back(pos);
    geom->setVertexArray(vertex.get());

    osg::ref_ptr<osg::Vec4Array>  color = new osg::Vec4Array();
    color->push_back(osg::Vec4(1.0f, 0.0f, 0.0f, 1.0f) );
    geom->setColorArray(color, osg::Array::BIND_OVERALL);

    geom->getOrCreateStateSet()->setAttribute(new osg::Point(6.0f), osg::StateAttribute::ON);
    geom->getOrCreateStateSet()->setMode(GL_POINT_SMOOTH, osg::StateAttribute::ON);
    geom->addPrimitiveSet(new osg::DrawArrays(osg::PrimitiveSet::POINTS, 0, 1));
    geom->getOrCreateStateSet()->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF | osg::StateAttribute::OVERRIDE);

    return geom;
}
