#include "myNodeVisitor.h"
#include "mainwindow.h"

#include <osg/Texture2D>
#include <osg/ProxyNode>
#include <osg/PagedLOD>
#include <osgDB/ReadFile>
#include <osgDB/WriteFile>

myNodeVisitor::myNodeVisitor()
{
    opt->setOptionString("WriteImageHint=IncludeFile");
}

void myNodeVisitor::apply(osg::Geode &node)
{
    // GeodeGeometry״㡢ߡ棩
    for(uint i = 0; i<node.getNumDrawables(); i++){
        osg::Geometry* pGeo = node.getDrawable(i)->asGeometry();
        if(pGeo){
            osg::Vec3Array* vertex = dynamic_cast<osg::Vec3Array*>(pGeo->getVertexArray());
            if (!vertex){
                continue;
            }
            for (int i = 0; i < vertex->size(); i++){
                // ĵ
                int index = pointsNearVertex((*vertex)[i]);
                if (index != -1)
                {
                    (*vertex)[i].z() = vertexArray->at(index).z();
                    affected = true;
                }
            }
        }
    }

    if (affected && !node.getName().empty()){
        // PagedLODĴʽһ
        newFileName = QString::fromStdString(node.getName()).split('.').first() + "~.osgb";
        node.accept(*tv);
        node.setName(newFileName.toStdString());
        // дļ
        osgDB::writeNodeFile(node, (workingDir + '/' + newFileName).toStdString(), opt);
        childAffected = true;
    }
}

void myNodeVisitor::apply(osg::Group &node)
{
    bool groupAffected = false;
    for(uint i = 0; i<node.getNumChildren(); i++){
        childAffected = false;
        node.getChild(i)->accept(*this);
        if(childAffected){
            groupAffected = true;
        }
    }

    if(groupAffected && !node.getName().empty()){
        // PagedLODĴʽһ
        newFileName = QString::fromStdString(node.getName()).split('.').first() + "~.osgb";
        node.accept(*tv);
        node.setName(newFileName.toStdString());
        // дļ
        osgDB::writeNodeFile(node, (workingDir + '/' + newFileName).toStdString(), opt);
        childAffected = true;
    }
}

void myNodeVisitor::apply(osg::PagedLOD &node)
{
    affected = false;
    childAffected = false;
    for(uint i = 0; i < node.getNumChildren(); i++){
        // ܻ̿޸ļ
        node.getChild(i)->accept(*this);
    }

//    // ʹaffectedΪжϣԾܵؼټ, affectedļᱻ޸
//    if(!affected && ! childAffected){
//        return;
//    }

    // ı޸ļô
    workingDir = QString::fromStdString(node.getDatabasePath());
    //        for(uint i = 0; i < node.getNumFileNames(); i++){
    for(uint i = 1; i < node.getNumFileNames(); i++){
        // LODļеļиϸһ㣬ҲҪ
        childAffected = false;
        // ļԼ
        osg::ref_ptr<osg::Node> fileNode = osgDB::readNodeFile(workingDir.toStdString() + '/' + node.getFileName(i));
        if(!fileNode.valid()){
            continue;
        }
        // fileNodeļһڵ㱻޸ʱԼʱ¶Ӧļ
        fileNode->setName(node.getFileName(i));
        // Ҫ
        fileNode->accept(*tv);
        fileNode->accept(*this);
        if(childAffected){
            //ļ
            node.setFileName(i, newFileName.toStdString());
            // ʱҲӦӦ޸pLODӦļ
        }
    }

    if(!node.getName().empty()){
        newFileName = QString::fromStdString(node.getName()).split('.').first() + "~.osgb";
        node.accept(*tv);
        node.setName(newFileName.toStdString());
        // дļ
        osgDB::writeNodeFile(node, (workingDir + '/' + newFileName).toStdString(), opt);
    }

    // Ϊ˵Ķӣļ޸
    childAffected = true;
}

bool myNodeVisitor::pointInPolygon(osg::Vec3 &point)
{
    for(osg::ref_ptr<osg::Vec3Array> boundary: boundaryList){
        int i, j, nvert = boundary->size();
        bool c = false;

        // ߷
        for (i = 0, j = nvert - 1; i < nvert; j = i++) {
            if (((boundary->at(i).y() >= point.y()) != (boundary->at(j).y() >= point.y())) &&
                    (point.x() <= (boundary->at(j).x() - boundary->at(i).x()) * (point.y() - boundary->at(i).y()) / (boundary->at(j).y() - boundary->at(i).y()) + boundary->at(i).x())
                    )
                c = !c;
        }
        if(c && point.z() > boundary->at(0).z()){
            return true;
        }
    }
    return false;
}

int myNodeVisitor::pointsNearVertex(osg::Vec3 &point)
{
    for(int i = 0; i < vertexArray->size(); i++){
        if(pow(point.x() - vertexArray->at(i).x(), 2) + pow(point.y() - vertexArray->at(i).y(), 2) < 4 && point.z() > vertexArray->at(i).z()){
            return i;
        }
    }
    return -1;
}
